@@ -120,10 +120,6 @@ struct _MetaDisplay
   Atom atom_net_client_list_stacking;
   Atom atom_net_wm_state_skip_taskbar;
   Atom atom_net_wm_state_skip_pager;
-  Atom atom_win_workspace;
-  Atom atom_win_layer;
-  Atom atom_win_protocols;
-  Atom atom_win_supporting_wm_check;
   Atom atom_net_wm_icon_name;
   Atom atom_net_wm_icon;
   Atom atom_net_wm_icon_geometry;
@@ -134,7 +130,6 @@ struct _MetaDisplay
   Atom atom_net_active_window;
   Atom atom_metacity_restart_message;
   Atom atom_net_wm_strut;
-  Atom atom_win_hints;
   Atom atom_metacity_reload_theme_message;
   Atom atom_metacity_set_keybindings_message;
   Atom atom_net_wm_state_hidden;
@@ -172,6 +167,8 @@ struct _MetaDisplay
   Atom atom_gnome_panel_action;
   Atom atom_gnome_panel_action_main_menu;
   Atom atom_gnome_panel_action_run_dialog;
+  Atom atom_metacity_sentinel;
+  Atom atom_net_wm_strut_partial;
   
   /* This is the actual window from focus events,
    * not the one we last set
@@ -182,11 +179,6 @@ struct _MetaDisplay
    */
   MetaWindow *expected_focus_window;
 
-  /* Most recently focused list. Always contains all
-   * live windows.
-   */
-  GList *mru_list;  
-
   guint static_gravity_works : 1;
   
   /*< private-ish >*/
@@ -205,7 +197,6 @@ struct _MetaDisplay
   Window no_focus_window;
   
   /* for double click */
-  int double_click_time;
   Time last_button_time;
   Window last_button_xwindow;
   int last_button_num;
@@ -237,18 +228,21 @@ struct _MetaDisplay
   int         grab_button;
   int         grab_initial_root_x;
   int         grab_initial_root_y;
-  int         grab_current_root_x;
-  int         grab_current_root_y;
   int         grab_latest_motion_x;
   int         grab_latest_motion_y;
   gulong      grab_mask;
   guint       grab_have_pointer : 1;
   guint       grab_have_keyboard : 1;
   MetaRectangle grab_initial_window_pos;
-  MetaRectangle grab_current_window_pos;
   MetaResizePopup *grab_resize_popup;
   GTimeVal    grab_last_moveresize_time;
   Time        grab_motion_notify_time;
+
+  /* we use property updates as sentinels for certain window focus events
+   * to avoid some race conditions on EnterNotify events
+   */
+  int         sentinel_counter;
+
 #ifdef HAVE_XKB
   int         xkb_base_event_type;
 #endif
@@ -435,9 +429,15 @@ gboolean meta_grab_op_is_resizing (MetaGrabOp op);
 gboolean meta_rectangle_intersect (MetaRectangle *src1,
                                    MetaRectangle *src2,
                                    MetaRectangle *dest);
+gboolean meta_rectangle_equal (const MetaRectangle *src1,
+                               const MetaRectangle *src2);
 
 void meta_display_devirtualize_modifiers (MetaDisplay        *display,
                                           MetaVirtualModifier modifiers,
                                           unsigned int       *mask);
 
+void meta_display_increment_focus_sentinel (MetaDisplay *display);
+void meta_display_decrement_focus_sentinel (MetaDisplay *display);
+gboolean meta_display_focus_sentinel_clear (MetaDisplay *display);
+
 #endif
